#region usings
using System;
using System.ComponentModel.Composition;

using VVVV.PluginInterfaces.V1;
using VVVV.PluginInterfaces.V2;
using VVVV.Utils.VColor;
using VVVV.Utils.VMath;

using VVVV.Core.Logging;
#endregion usings

namespace VVVV.Nodes
{
	#region PluginInfo
	[PluginInfo(Name = "Lerp", Category = "Color", Help = "Generate interpolated color from table", Tags = "")]
	#endregion PluginInfo
	public class ColorLerpNode : IPluginEvaluate
	{
		#region fields & pins
		[Input("Input")]
		public ISpread<float> FInput;

		[Input("Control Points")]
		public ISpread<float> FControl;

		[Input("R")]
		public ISpread<float> FR;

		[Input("G")]
		public ISpread<float> FG;

		[Input("B")]
		public ISpread<float> FB;

		[Input("A")]
		public ISpread<float> FA;

		[Output("Output")]
		public ISpread<RGBAColor> FOutput;

		[Import()]
		public ILogger FLogger;
		#endregion fields & pins

		//called when data for any output pin is requested
		public void Evaluate(int SpreadMax)
		{
			int jmax = FControl.SliceCount;
			int cmax = FControl.SliceCount - 1;
			
			FOutput.SliceCount = FInput.SliceCount;
			
			for ( int i = 0; i < FInput.SliceCount; i++ )
			{
				if ( FInput[i] <= FControl[0] )
				{
					// below minimum slice
					FOutput[i] = new RGBAColor(FR[0],FG[0],FB[0],FA[0]);
				}
				else if ( FInput[i] >= FControl[cmax] )
				{
					// past max slice
					FOutput[i] = new RGBAColor(FR[cmax],FG[cmax],FB[cmax],FA[cmax]);
				}
				else
				{
					for ( int j = 0; j < jmax; j++ )
					{
						if ( FInput[i] >= FControl[j] && FInput[i] < FControl[j+1] )
						{
							// interpolate between these values
							
							float inc = FInput[i] - FControl[j];
							float range = FControl[j+1] - FControl[j];
							float scale = inc / range;

							float red = FR[j] + (FR[j+1] - FR[j]) * scale;
							float green = FG[j] + (FG[j+1] - FG[j]) * scale;
							float blue = FB[j] + (FB[j+1] - FB[j]) * scale;
							float alpha = FA[j] + (FA[j+1] - FA[j]) * scale;

							FOutput[i] = new RGBAColor(red,green,blue,alpha);
							break;
						}
					}
				}
			}
		}
	}
}
